/* made by profil 2011-12-29.
 * Added:
 *    * Battery and temperature status for Linux and FreeBSD
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <time.h>
#include <X11/Xlib.h>
#include <sys/types.h>
#include <sys/sysctl.h>

#define SIZE 533
#define F_BATT "/sys/class/power_supply/BAT1/capacity"
#define F_TMPER	"/sys/bus/acpi/drivers/thermal/LNXTHERM:00/thermal_zone/temp"

#define BAT_MIB_LEN	4
#define TEMP_MIB_LEN	5
#define AC_MIB_LEN	3

#define BATBUF_LEN	16
#define DATETIME_LEN	32
#define TEMPER_LEN	8
#define STATUS_LEN	200

static Display *dpy;

void
setstatus(char *str)
{
	XStoreName(dpy, DefaultRootWindow(dpy), str);
	XSync(dpy, False);
}

int
getdatetime(char *buf)
{
	time_t result;
	struct tm *resulttm;

	result = time(NULL);
	resulttm = localtime(&result);

	if (resulttm == NULL) {
		fprintf(stderr, "Error getting localtime.\n");
		return 1;
	}

	if (!strftime(buf, DATETIME_LEN - 1, "%D | %l:%M:%S%p", resulttm)) {
		fprintf(stderr, "strftime is 0. Buffer too small?\n");
		return 1;
	}
	
	return 0;
}

#ifdef __FreeBSD__
int
init_sysctl(char *name, size_t miblen, int *mib)
{
	return sysctlnametomib(name, mib, &miblen);
}
#endif

int
gettemp(int *mib, char *buf)
{
	int c_temp;

#ifdef __FreeBSD__
	int kelvin_temp;
	size_t len = sizeof(kelvin_temp);

	if (sysctl(mib, TEMP_MIB_LEN, &kelvin_temp, &len, NULL, 0)) {
		perror("Failed to get system temperature");
		return -1;
	}

	c_temp = (kelvin_temp - 2732) / 10;

	snprintf(buf, TEMPER_LEN, "%dC", c_temp);

#elif __linux__

	FILE *tfile = fopen(F_TMPER, "r");

	if (tfile == NULL) {
		fprintf(stderr, "Could not open file %s: %s\n",
                      F_TMPER, strerror(errno));
		return -1;
	}


	if (fscanf(tfile, "%g", &c_temp) == EOF) {
		fprintf(stderr, "Failed to read temperature\n");
		fclose(tfile);
		return -1;
	}

	fclose(tfile);
	
	snprintf(buf, TEMPER_LEN, "%dC", c_temp / 1000);
#endif

	return 0;
}

int
getbat(int *mib, int *power, char *buf)
{

#ifdef __linux__
	FILE *batstat;
	int batval;

	if ((batstat = fopen(F_BATT, "r")) == NULL) {
		fprintf(stderr, "Could not open file %s: %s\n",
                      F_BATT, strerror(errno));
		return -1;
	}

	if (fscanf(batstat, "%d", &batval) == EOF) {
		fprintf(stderr, "Failed to read battery status\n");
		fclose(batstat);
		return -1;
	}

	snprintf(buf, BATBUF_LEN, "BAT %d", batval);

	fclose(batstat);

#elif __FreeBSD__

	int batlife;
	int plugged_in, plugged_in_err;

	size_t batlife_len = sizeof(batlife);

	if (sysctl(mib, BAT_MIB_LEN, &batlife, &batlife_len, NULL, 0)) {
		perror("Failed to get battery status");
		return -1;
	}

	plugged_in_err = sysctl(power, AC_MIB_LEN, &plugged_in,
		&batlife_len, NULL, 0);

	if (plugged_in_err || !plugged_in) {
		snprintf(buf, BATBUF_LEN, "BAT %d", batlife);
	} else {
		snprintf(buf, BATBUF_LEN, "AC, BAT %d", batlife);
	}

#endif
	return 0;
}

int
main()
{
	int mib_bat[BAT_MIB_LEN],
	    mib_temp[TEMP_MIB_LEN],
	    mib_ac[AC_MIB_LEN];

	char bat_buf[BATBUF_LEN],
	     datetime[DATETIME_LEN],
	     temp_buf[TEMPER_LEN],
	     status[STATUS_LEN];

	if (!(dpy = XOpenDisplay(NULL))) {
		fprintf(stderr, "Cannot open display.\n");
		return 1;
	}

#ifdef __FreeBSD__
	init_sysctl("hw.acpi.battery.life", BAT_MIB_LEN, mib_bat);
	init_sysctl("hw.acpi.thermal.tz0.temperature", TEMP_MIB_LEN, mib_temp);
	init_sysctl("hw.acpi.acline", AC_MIB_LEN, mib_ac);
#endif
	
	while (1) {
		getdatetime(datetime);
		getbat(mib_bat, mib_ac, bat_buf);
		gettemp(mib_temp, temp_buf);

		snprintf(status, 200, "%s%% | %s | %s", bat_buf, temp_buf, datetime);
	
		setstatus(status);
		sleep(1);
	}

	XCloseDisplay(dpy);

	return 0;
}
